@@ -31,14 +31,18 @@ import threading
 
 from StringIO import StringIO
 from collections import defaultdict
+from urlparse import urljoin
+from contextlib import contextmanager
 
 
-from couchdb.client import Server
+from couchdb.client import Server, Database
 from couchdb.http import (
     ResourceConflict,
     ResourceNotFound,
     ServerError,
     Session,
+    urljoin as couch_urljoin,
+    Resource,
 )
 from u1db import query_parser, vectorclock
 from u1db.errors import (
@@ -331,6 +335,23 @@ class MultipartWriter(object):
                 self.headers[name] = value
 
 
+@contextmanager
+def couch_server(url):
+    """
+    Provide a connection to a couch server and cleanup after use.
+
+    For database creation and deletion we use an ephemeral connection to the
+    couch server. That connection has to be properly closed, so we provide it
+    as a context manager.
+
+    :param url: The URL of the Couch server.
+    :type url: str
+    """
+    session = Session(timeout=COUCH_TIMEOUT)
+    server = Server(url=url, session=session)
+    yield server
+
+
 class CouchDatabase(CommonBackend):
     """
     A U1DB implementation that uses CouchDB as its persistence layer.
@@ -340,6 +361,7 @@ class CouchDatabase(CommonBackend):
     MAX_GET_DOCS_THREADS = 20
 
     update_handler_lock = defaultdict(threading.Lock)
+    sync_info_lock = defaultdict(threading.Lock)
 
     class _GetDocThread(threading.Thread):
         """
@@ -353,7 +375,7 @@ class CouchDatabase(CommonBackend):
                      release_fun):
             """
             :param db: The database from where to get the document.
-            :type db: u1db.Database
+            :type db: CouchDatabase
             :param doc_id: The doc_id of the document to be retrieved.
             :type doc_id: str
             :param check_for_conflicts: Whether the get_doc() method should
@@ -380,7 +402,7 @@ class CouchDatabase(CommonBackend):
             self._release_fun()
 
     @classmethod
-    def open_database(cls, url, create, ensure_ddocs=False):
+    def open_database(cls, url, create, replica_uid=None, ensure_ddocs=False):
         """
         Open a U1DB database using CouchDB as backend.
 
@@ -388,6 +410,8 @@ class CouchDatabase(CommonBackend):
         :type url: str
         :param create: should the replica be created if it does not exist?
         :type create: bool
+        :param replica_uid: an optional unique replica identifier
+        :type replica_uid: str
         :param ensure_ddocs: Ensure that the design docs exist on server.
         :type ensure_ddocs: bool
 
@@ -400,16 +424,17 @@ class CouchDatabase(CommonBackend):
             raise InvalidURLError
         url = m.group(1)
         dbname = m.group(2)
-        server = Server(url=url)
-        try:
-            server[dbname]
-        except ResourceNotFound:
-            if not create:
-                raise DatabaseDoesNotExist()
-        return cls(url, dbname, ensure_ddocs=ensure_ddocs)
+        with couch_server(url) as server:
+            try:
+                server[dbname]
+            except ResourceNotFound:
+                if not create:
+                    raise DatabaseDoesNotExist()
+                server.create(dbname)
+        return cls(
+            url, dbname, replica_uid=replica_uid, ensure_ddocs=ensure_ddocs)
 
-    def __init__(self, url, dbname, replica_uid=None, full_commit=True,
-                 session=None, ensure_ddocs=True):
+    def __init__(self, url, dbname, replica_uid=None, ensure_ddocs=True):
         """
         Create a new Couch data container.
 
@@ -419,31 +444,23 @@ class CouchDatabase(CommonBackend):
         :type dbname: str
         :param replica_uid: an optional unique replica identifier
         :type replica_uid: str
-        :param full_commit: turn on the X-Couch-Full-Commit header
-        :type full_commit: bool
-        :param session: an http.Session instance or None for a default session
-        :type session: http.Session
         :param ensure_ddocs: Ensure that the design docs exist on server.
         :type ensure_ddocs: bool
         """
         # save params
         self._url = url
-        self._full_commit = full_commit
-        if session is None:
-            session = Session(timeout=COUCH_TIMEOUT)
-        self._session = session
+        self._session = Session(timeout=COUCH_TIMEOUT)
         self._factory = CouchDocument
         self._real_replica_uid = None
         # configure couch
-        self._server = Server(url=self._url,
-                              full_commit=self._full_commit,
-                              session=self._session)
         self._dbname = dbname
+        self._database = Database(
+            urljoin(self._url, self._dbname),
+            self._session)
         try:
-            self._database = self._server[self._dbname]
+            self._database.info()
         except ResourceNotFound:
-            self._server.create(self._dbname)
-            self._database = self._server[self._dbname]
+            raise DatabaseDoesNotExist()
         if replica_uid is not None:
             self._set_replica_uid(replica_uid)
         if ensure_ddocs:
@@ -482,7 +499,8 @@ class CouchDatabase(CommonBackend):
         """
         Delete a U1DB CouchDB database.
         """
-        del(self._server[self._dbname])
+        with couch_server(self._url) as server:
+            del(server[self._dbname])
 
     def close(self):
         """
@@ -494,10 +512,15 @@ class CouchDatabase(CommonBackend):
         self._url = None
         self._full_commit = None
         self._session = None
-        self._server = None
         self._database = None
         return True
 
+    def __del__(self):
+        """
+        Close the database upon garbage collection.
+        """
+        self.close()
+
     def _set_replica_uid(self, replica_uid):
         """
         Force the replica uid to be set.
@@ -540,6 +563,8 @@ class CouchDatabase(CommonBackend):
 
     _replica_uid = property(_get_replica_uid, _set_replica_uid)
 
+    replica_uid = property(_get_replica_uid)
+
     def _get_generation(self):
         """
         Return the current generation.
@@ -833,7 +858,7 @@ class CouchDatabase(CommonBackend):
             # Date.prototype.getTime() which was used before inside a couchdb
             # update handler.
             (int(time.time() * 1000),
-            self._allocate_transaction_id()))
+             self._allocate_transaction_id()))
         # build the couch document
         couch_doc = {
             '_id': doc.doc_id,
@@ -853,9 +878,9 @@ class CouchDatabase(CommonBackend):
         envelope.close()
         # try to save and fail if there's a revision conflict
         try:
-            self._database.resource.put_json(
+            resource = self._new_resource()
+            resource.put_json(
                 doc.doc_id, body=buf.getvalue(), headers=envelope.headers)
-            self._renew_couch_session()
         except ResourceConflict:
             raise RevisionConflict()
 
@@ -1411,7 +1436,7 @@ class CouchDatabase(CommonBackend):
         # strptime here by evaluating the conversion of an arbitrary date.
         # This will not be needed when/if we switch from python-couchdb to
         # paisley.
-        time.strptime('Mar 4 1917', '%b %d %Y')
+        time.strptime('Mar 8 1917', '%b %d %Y')
         # spawn threads to retrieve docs
         threads = []
         for doc_id in doc_ids:
@@ -1427,14 +1452,19 @@ class CouchDatabase(CommonBackend):
                 continue
             yield t._doc
 
-    def _renew_couch_session(self):
+    def _new_resource(self, *path):
         """
-        Create a new couch connection session.
+        Return a new resource for accessing a couch database.
 
-        This is a workaround for #5448. Will not be needed once bigcouch is
-        merged with couchdb.
+        :return: A resource for accessing a couch database.
+        :rtype: couchdb.http.Resource
         """
-        self._database.resource.session = Session(timeout=COUCH_TIMEOUT)
+        # Workaround for: https://leap.se/code/issues/5448
+        url = couch_urljoin(self._database.resource.url, *path)
+        resource = Resource(url, Session(timeout=COUCH_TIMEOUT))
+        resource.credentials = self._database.resource.credentials
+        resource.headers = self._database.resource.headers.copy()
+        return resource
 
 
 class CouchSyncTarget(CommonSyncTarget):
@@ -1489,9 +1519,9 @@ class CouchServerState(ServerState):
         :return: The CouchDatabase object.
         :rtype: CouchDatabase
         """
-        return CouchDatabase.open_database(
-            self._couch_url + '/' + dbname,
-            create=False,
+        return CouchDatabase(
+            self._couch_url,
+            dbname,
             ensure_ddocs=False)
 
     def ensure_database(self, dbname):
@@ -1508,8 +1538,8 @@ class CouchServerState(ServerState):
         :param dbname: The name of the database to ensure.
         :type dbname: str
 
-        :return: The CouchDatabase object and the replica uid.
-        :rtype: (CouchDatabase, str)
+        :raise Unauthorized: Always, because Soledad server is not allowed to
+                             create databases.
         """
         raise Unauthorized()
 
@@ -1519,6 +1549,9 @@ class CouchServerState(ServerState):
 
         :param dbname: The name of the database to delete.
         :type dbname: str
+
+        :raise Unauthorized: Always, because Soledad server is not allowed to
+                             delete databases.
         """
         raise Unauthorized()
 
